﻿using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UF.AssessmentProject.Controllers;
using UF.AssessmentProject.Helper;
using Xunit;

namespace TestProject1
{
    public class ControllerTest
    {
        TransactionController _controller = new TransactionController();

        [Fact]
        public void Expired()
        {
            var rq = new UF.AssessmentProject.Model.Transaction.RequestMessage()
            {
                partnerkey = "FAKEGOOGLE",
                partnerpassword = "FG-00001",
                partnerrefno = "RkFLRVBBU1NXT1JEMTIzNA==",
                sig = "24XYSmvKGH9I9Y5FLvSsId2MPtjkvog7U5JLhE3m30A=",
                timestamp = "2013-11-22T02:11:22.0000000Z",
                totalamount = 1000,
                items = new List<UF.AssessmentProject.Model.Transaction.itemdetail>()
                {
                    new UF.AssessmentProject.Model.Transaction.itemdetail()
                    {
                        name = "i-00001",
                        partneritemref = "Pen",
                        qty = 5,
                        unitprice = 2
                    },
                    new UF.AssessmentProject.Model.Transaction.itemdetail()
                    {
                        name = "i-00002",
                        partneritemref = "Ruler",
                        qty = 1,
                        unitprice = 100
                    }
                }
            };
            ActionResult<UF.AssessmentProject.Model.Transaction.ResponseMessage> rs = _controller.SubmitTRansaction(rq);
            var actual = (OkObjectResult)rs.Result;
            var resultmessage = actual.Value as UF.AssessmentProject.Model.Transaction.ResponseMessage;
            Assert.Equal("Expired!", resultmessage.resultmessage);
        }

        [Fact]
        public void Access_denied()
        {
            
            var rq = new UF.AssessmentProject.Model.Transaction.RequestMessage()
            {
                partnerkey = "aaa",
                partnerpassword = "FG-00001",
                partnerrefno = "RkFLRVBBU1NXT1JEMTIzNA==",
                totalamount = 1000,
                items = new List<UF.AssessmentProject.Model.Transaction.itemdetail>()
                {
                    new UF.AssessmentProject.Model.Transaction.itemdetail()
                    {
                        name = "i-00001",
                        partneritemref = "Pen",
                        qty = 5,
                        unitprice = 2
                    },
                    new UF.AssessmentProject.Model.Transaction.itemdetail()
                    {
                        name = "i-00002",
                        partneritemref = "Ruler",
                        qty = 1,
                        unitprice = 100
                    }
                }
            };
            rq.timestamp = DateTime.Now.ToString();
            var rawsign = DateTime.Parse(rq.timestamp).ToString("yyyyMMddHHmmss") + rq.partnerkey + rq.partnerrefno + rq.totalamount + rq.partnerpassword;
            string sig = CommonUtils.ComputeSha256Hash((CommonUtils.EncodeBase64(rawsign)));
            rq.sig = sig;

            ActionResult<UF.AssessmentProject.Model.Transaction.ResponseMessage> rs = _controller.SubmitTRansaction(rq);
            var actual = (OkObjectResult)rs.Result;
            var resultmessage = actual.Value as UF.AssessmentProject.Model.Transaction.ResponseMessage;
            Assert.Equal("Access Denied!", resultmessage.resultmessage);
        }
    }
}
